// HIGHLANDS
// by Ghoul

// Description: Almost exactly like Highlands from Age of Kings (AoE2).
// Players start on they're own little "island" with rivers between other
// players.  Shallows exist between adjacent players making for great
// defensive choke points.  Resources are normal in supply, trees are
// perhaps lightly more abundant than in most maps.

// Main entry point for random map script
void main(void)
{

   // Text
   rmSetStatusText("",0.01);

   // Set size.
   int playerTiles=9000;
   if(cMapSize == 1)
   {
      playerTiles = 11700;
      rmEchoInfo("Large map");
   }
   int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
   rmEchoInfo("Map size="+size+"m x "+size+"m");
   rmSetMapSize(size, size);

   // Set up default water.
   rmSetSeaLevel(1.0);
   rmSetSeaType("Greek River");

   // Init map.
   rmTerrainInitialize("water");

   // Define some classes.
   int classIsland=rmDefineClass("island");
   int classBonusIsland=rmDefineClass("bonus island");
   int classPlayerCore=rmDefineClass("player core");
   int classPlayer=rmDefineClass("player");
   rmDefineClass("corner");
   rmDefineClass("starting settlement");
   rmDefineClass("center");
   
   // Create a edge of map constraint.
   int edgeConstraint=rmCreateBoxConstraint("edge of map", rmXTilesToFraction(1), rmZTilesToFraction(1), 1.0-rmXTilesToFraction(1), 1.0-rmZTilesToFraction(1));
   int playerEdgeConstraint=rmCreateBoxConstraint("player edge of map", rmXTilesToFraction(8), rmZTilesToFraction(8), 1.0-rmXTilesToFraction(8), 1.0-rmZTilesToFraction(8), 0.01);
   int centerConstraint=rmCreateClassDistanceConstraint("stay away from center", rmClassID("center"), 40.0);

   // Player area constraint.
   int islandConstraint=rmCreateClassDistanceConstraint("stay away from islands", classIsland, 20.0);
   int coreBonusConstraint=rmCreateClassDistanceConstraint("core v bonus island", classPlayerCore, 50.0);
   int playerConstraint=rmCreateClassDistanceConstraint("bonus Settlement stay away from players", classPlayer, 10);

   // Bonus area constraint.
   int bonusIslandConstraint=rmCreateClassDistanceConstraint("avoid bonus island", classBonusIsland, 15.0);
   int bonusIslandEdgeConstraint=rmCreateBoxConstraint("bonus island avoids edge", rmXTilesToFraction(30), rmZTilesToFraction(30), 1.0-rmXTilesToFraction(30), 1.0-rmZTilesToFraction(30)); 

   // corner constraint.
   int cornerConstraint=rmCreateClassDistanceConstraint("stay away from corner", rmClassID("corner"), 15.0);
   int cornerOverlapConstraint=rmCreateClassDistanceConstraint("don't overlap corner", rmClassID("corner"), 2.0);

   // Settlement constraint.
   int avoidSettlement=rmCreateTypeDistanceConstraint("avoid settlement", "AbstractSettlement", 50.0);
   int shortAvoidSettlement=rmCreateTypeDistanceConstraint("short avoid settlement", "AbstractSettlement", 10.0);
   int farAvoidSettlement=rmCreateTypeDistanceConstraint("TCs avoid TCs by long distance", "AbstractSettlement", 50.0);

   // Far starting settlement constraint.
   int farStartingSettleConstraint=rmCreateClassDistanceConstraint("far start settle", rmClassID("starting settlement"), 50.0);

   // Tower constraint.
   int avoidTower=rmCreateTypeDistanceConstraint("avoid tower", "tower", 25.0);

   // Gold
   int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 30.0);
   int shortAvoidGold=rmCreateTypeDistanceConstraint("short avoid gold", "gold", 10.0);

   // herdable
   int avoidHerdable=rmCreateTypeDistanceConstraint("avoid herdable", "herdable", 20.0);

   // Bonus huntable
   int classBonusHuntable=rmDefineClass("bonus huntable");
   int avoidBonusHuntable=rmCreateClassDistanceConstraint("avoid bonus huntable", classBonusHuntable, 15.0);
   int avoidFood=rmCreateTypeDistanceConstraint("avoid other food sources", "food", 6.0);
   int avoidFoodFar=rmCreateTypeDistanceConstraint("avoid food by more", "food", 20.0);
   int avoidPredator=rmCreateTypeDistanceConstraint("avoid predator", "animalPredator", 20.0);

   // Avoid impassable land
   int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "Land", false, 10.0);
   int shortAvoidImpassableLand=rmCreateTerrainDistanceConstraint("short avoid impassable land", "Land", false, 5.0);
   int tinyAvoidImpassableLand=rmCreateTerrainDistanceConstraint("tiny avoid impassable land", "Land", false, 2.0);
   int avoidAll=rmCreateTypeDistanceConstraint("avoid all", "all", 6.0);
   int farAvoidImpassableLand=rmCreateTerrainDistanceConstraint("far avoid impassable land", "land", false, 20.0);

      // Stay near shore
   int nearShore=rmCreateTerrainMaxDistanceConstraint("near shore", "water", true, 6.0);


   // note - try and trim down on the number of these. Many must be redundant.


 // -------------Define objects
   // Close Objects

   int startingSettlementID=rmCreateObjectDef("Starting settlement");
   rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
   rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
   rmSetObjectDefMinDistance(startingSettlementID, 0.0);
   rmSetObjectDefMaxDistance(startingSettlementID, 0.0);

   // towers avoid other towers
   int startingTowerID=rmCreateObjectDef("Starting tower");
   rmAddObjectDefItem(startingTowerID, "tower", 1, 0.0);
   rmSetObjectDefMinDistance(startingTowerID, 22.0);
   rmSetObjectDefMaxDistance(startingTowerID, 28.0);
   rmAddObjectDefConstraint(startingTowerID, avoidTower);

   // gold avoids gold
   int startingGoldID=rmCreateObjectDef("Starting gold");
   rmAddObjectDefItem(startingGoldID, "Gold mine small", 1, 0.0);
   rmSetObjectDefMinDistance(startingGoldID, 20.0);
   rmSetObjectDefMaxDistance(startingGoldID, 25.0);
   rmAddObjectDefConstraint(startingGoldID, avoidGold);
   rmAddObjectDefConstraint(startingGoldID, avoidImpassableLand);

   int closeGoatsID=rmCreateObjectDef("close Goats");
   rmAddObjectDefItem(closeGoatsID, "goat", rmRandInt(0,2), 2.0);
   rmSetObjectDefMinDistance(closeGoatsID, 25.0);
   rmSetObjectDefMaxDistance(closeGoatsID, 30.0);
   rmAddObjectDefConstraint(closeGoatsID, avoidImpassableLand);
   rmAddObjectDefConstraint(closeGoatsID, avoidFood);

   int closeDeerID=rmCreateObjectDef("close deer");
   rmAddObjectDefItem(closeDeerID, "deer", rmRandInt(3,8), 4.0);
   rmSetObjectDefMinDistance(closeDeerID, 25.0);
   rmSetObjectDefMaxDistance(closeDeerID, 30.0);
   rmAddObjectDefConstraint(closeDeerID, avoidImpassableLand);
   rmAddObjectDefConstraint(closeDeerID, avoidFood);

   int closeElkID=rmCreateObjectDef("close elk");
   if(rmRandFloat(0, 1)<0.5)
      rmAddObjectDefItem(closeElkID, "elk", 2, 1.0);
   else
      rmAddObjectDefItem(closeElkID, "elk", 3, 4.0);
   rmSetObjectDefMinDistance(closeElkID, 30.0);
   rmSetObjectDefMaxDistance(closeElkID, 50.0);
   rmAddObjectDefConstraint(closeElkID, avoidImpassableLand);
   
   int stragglerTreeID=rmCreateObjectDef("straggler tree");
   rmAddObjectDefItem(stragglerTreeID, "pine", 1, 0.0);
   rmSetObjectDefMinDistance(stragglerTreeID, 12.0);
   rmSetObjectDefMaxDistance(stragglerTreeID, 15.0);
   rmAddObjectDefConstraint(stragglerTreeID, avoidImpassableLand);

   // medium objects

   // Text
   rmSetStatusText("",0.20);

   // gold avoids gold and Settlements
   int mediumGoldID=rmCreateObjectDef("medium gold");
   rmAddObjectDefItem(mediumGoldID, "Gold mine", 1, 0.0);
   rmSetObjectDefMinDistance(mediumGoldID, 40.0);
   rmSetObjectDefMaxDistance(mediumGoldID, 60.0);
   rmAddObjectDefConstraint(mediumGoldID, avoidGold);
   rmAddObjectDefConstraint(mediumGoldID, edgeConstraint);
   rmAddObjectDefConstraint(mediumGoldID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(mediumGoldID, avoidImpassableLand);

   int mediumGoatsID=rmCreateObjectDef("medium goats");
   rmAddObjectDefItem(mediumGoatsID, "goat", rmRandInt(2,3), 4.0);
   rmSetObjectDefMinDistance(mediumGoatsID, 50.0);
   rmSetObjectDefMaxDistance(mediumGoatsID, 70.0);
   rmAddObjectDefConstraint(mediumGoatsID, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumGoatsID, avoidFoodFar);
   rmAddObjectDefConstraint(mediumGoatsID, farStartingSettleConstraint);

   // For this map, pick how many deer/elk in a grouping.  Assign this
   // to both elk and deer since we place them interchangeably per player.
   int numHuntable=rmRandInt(6, 10);

   int mediumDeerID=rmCreateObjectDef("medium deer");
   rmAddObjectDefItem(mediumDeerID, "deer", numHuntable, 4.0);
   rmSetObjectDefMinDistance(mediumDeerID, 40.0);
   rmSetObjectDefMaxDistance(mediumDeerID, 80.0);
   rmAddObjectDefConstraint(mediumDeerID, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumDeerID, farStartingSettleConstraint);

   int mediumElkID=rmCreateObjectDef("medium elk");
   rmAddObjectDefItem(mediumElkID, "elk", numHuntable, 4.0);
   rmSetObjectDefMinDistance(mediumElkID, 40.0);
   rmSetObjectDefMaxDistance(mediumElkID, 80.0);
   rmAddObjectDefConstraint(mediumElkID, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumElkID, farStartingSettleConstraint);

   // far objects

   // gold avoids gold, Settlements and TCs
   int farGoldID=rmCreateObjectDef("far gold");
   rmAddObjectDefItem(farGoldID, "Gold mine", 1, 0.0);
   rmSetObjectDefMinDistance(farGoldID, 80.0);
   rmSetObjectDefMaxDistance(farGoldID, 100.0);
   rmAddObjectDefConstraint(farGoldID, avoidGold);
   rmAddObjectDefConstraint(farGoldID, edgeConstraint);
   rmAddObjectDefConstraint(farGoldID, shortAvoidSettlement);
   rmAddObjectDefConstraint(farGoldID, avoidImpassableLand);
/*
   // gold avoids gold, Settlements and TCs
   int bonusGoldID=rmCreateObjectDef("gold on bonus islands");
   rmAddObjectDefItem(bonusGoldID, "Gold mine", 1, 0.0);
   rmSetObjectDefMinDistance(bonusGoldID, 90.0);
   rmSetObjectDefMaxDistance(bonusGoldID, 180.0);
   rmAddObjectDefConstraint(bonusGoldID, shortAvoidGold);
   rmAddObjectDefConstraint(bonusGoldID, edgeConstraint);
   rmAddObjectDefConstraint(bonusGoldID, shortAvoidSettlement);
   rmAddObjectDefConstraint(bonusGoldID, playerConstraint);
   rmAddObjectDefConstraint(bonusGoldID, avoidImpassableLand);
*/
   int farGoatsID=rmCreateObjectDef("far goats");
   rmAddObjectDefItem(farGoatsID, "goat", rmRandInt(1,2), 4.0);
   rmSetObjectDefMinDistance(farGoatsID, 80.0);
   rmSetObjectDefMaxDistance(farGoatsID, 150.0);
   rmAddObjectDefConstraint(farGoatsID, avoidHerdable);
   rmAddObjectDefConstraint(farGoatsID, shortAvoidImpassableLand);
  
   // pick bears or wolfs as predators
   // avoid TCs
   int farPredatorID=rmCreateObjectDef("far predator");
   float predatorSpecies=rmRandFloat(0, 1);
   if(predatorSpecies<0.5)   
      rmAddObjectDefItem(farPredatorID, "wolf", 2, 4.0);
   else
      rmAddObjectDefItem(farPredatorID, "bear", 3, 2.0);
   rmSetObjectDefMinDistance(farPredatorID, 50.0);
   rmSetObjectDefMaxDistance(farPredatorID, 100.0);
   rmAddObjectDefConstraint(farPredatorID, avoidPredator);
   rmAddObjectDefConstraint(farPredatorID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farPredatorID, shortAvoidImpassableLand);

   // try and place near water

   int numChicken=rmRandInt(6, 8);
   int farChickenID=rmCreateObjectDef("far chicken");
   rmAddObjectDefItem(farChickenID, "chicken", numChicken, 3.0);
   rmSetObjectDefMinDistance(farChickenID, 50.0);
   rmSetObjectDefMaxDistance(farChickenID, 150.0);
   rmAddObjectDefConstraint(farChickenID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farChickenID, nearShore);

   // Birds
   int farhawkID=rmCreateObjectDef("far hawks");
   rmAddObjectDefItem(farhawkID, "hawk", 1, 0.0);
   rmSetObjectDefMinDistance(farhawkID, 0.0);
   rmSetObjectDefMaxDistance(farhawkID, rmXFractionToMeters(0.5));

   // Relics avoid TCs
   int relicID=rmCreateObjectDef("relic");
   rmAddObjectDefItem(relicID, "relic", 1, 0.0);
   rmSetObjectDefMinDistance(relicID, 60.0);
   rmSetObjectDefMaxDistance(relicID, 150.0);
   rmAddObjectDefConstraint(relicID, edgeConstraint);
   rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic vs relic", "relic", 70.0));
   rmAddObjectDefConstraint(relicID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(relicID, shortAvoidImpassableLand);

   int randomTreeID=rmCreateObjectDef("random tree");
   rmAddObjectDefItem(randomTreeID, "pine", 1, 0.0);
   rmSetObjectDefMinDistance(randomTreeID, 0.0);
   rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(randomTreeID, rmCreateTypeDistanceConstraint("random tree", "all", 4.0));
   rmAddObjectDefConstraint(randomTreeID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(randomTreeID, shortAvoidSettlement);

   int randomTreeID2=rmCreateObjectDef("random tree 2");
   rmAddObjectDefItem(randomTreeID2, "pine", 1, 0.0);
   rmSetObjectDefMinDistance(randomTreeID2, 0.0);
   rmSetObjectDefMaxDistance(randomTreeID2, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(randomTreeID2, rmCreateTypeDistanceConstraint("random tree two", "all", 4.0));
   rmAddObjectDefConstraint(randomTreeID2, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(randomTreeID2, shortAvoidSettlement);

   // --------------------------------------------------------------------------------------------Done defining objects

   // Create areas
    rmPlacePlayersCircular(0.4, 0.45, rmDegreesToRadians(5.0));

   int centerID=rmCreateArea("center");
   rmSetAreaSize(centerID, 0.01, 0.01);
   rmSetAreaLocation(centerID, 0.5, 0.5);
   rmAddAreaToClass(centerID, rmClassID("center"));
   rmBuildArea(centerID);

    // Create player cores so bonus islands avoid them
    for(i=1; <cNumberPlayers)
   {
      // Create the area.
      int id=rmCreateArea("Player core"+i);
      rmSetAreaSize(id, rmAreaTilesToFraction(200), rmAreaTilesToFraction(200));
      rmAddAreaToClass(id, classPlayerCore);
      rmSetAreaCoherence(id, 1.0);
      rmSetAreaLocPlayer(id, i);

      rmBuildArea(id);
   }

   // Create connections
   int shallowsID=rmCreateConnection("shallows");
   rmSetConnectionType(shallowsID, cConnectPlayers, false, 1.0);
   rmSetConnectionWidth(shallowsID, rmRandInt(4,8), 2);
   rmSetConnectionWarnFailure(shallowsID, false);
   rmSetConnectionBaseHeight(shallowsID, 2.0);
   rmSetConnectionHeightBlend(shallowsID, 2.0);
   rmSetConnectionSmoothDistance(shallowsID, 3.0);
   rmAddConnectionTerrainReplacement(shallowsID, "riverGrassyA", "riverGrassyA");

   // Create extra connections
   int extraShallowsID=rmCreateConnection("extra shallows");
   if(cNumberPlayers < 5)
      rmSetConnectionType(extraShallowsID, cConnectAreas, false, 0.75);
   else if(cNumberPlayers < 7) 
      rmSetConnectionType(extraShallowsID, cConnectAreas, false, 0.30);
   rmSetConnectionWidth(extraShallowsID, 4, 8);
   rmSetConnectionWarnFailure(extraShallowsID, false); 
   rmSetConnectionBaseHeight(extraShallowsID, 2.0);
   rmSetConnectionHeightBlend(extraShallowsID, 2.0);
   rmSetConnectionSmoothDistance(extraShallowsID, 3.0);
   rmSetConnectionPositionVariance(extraShallowsID, -1.0); 
   rmAddConnectionTerrainReplacement(extraShallowsID, "riverGrassyA", "riverGrassyA"); 
   rmAddConnectionStartConstraint(extraShallowsID, coreBonusConstraint);
   rmAddConnectionEndConstraint(extraShallowsID, coreBonusConstraint);

  // Create team connections
   int teamShallowsID=rmCreateConnection("team shallows");
   rmSetConnectionType(teamShallowsID, cConnectAllies, false, 1.0);
   rmSetConnectionWarnFailure(teamShallowsID, false);
   rmSetConnectionWidth(teamShallowsID, 16, 2);
   rmSetConnectionBaseHeight(teamShallowsID, 2.0);
   rmSetConnectionHeightBlend(teamShallowsID, 2.0);
   rmSetConnectionSmoothDistance(teamShallowsID, 3.0);
   rmAddConnectionTerrainReplacement(teamShallowsID, "riverGrassyA", "riverGrassyA");
/*
   // Build up some bonus islands.
   int bonusCount = rmRandInt(2, 3);
   int bonusIsleSize = 1800;

   if(cNumberNonGaiaPlayers > 5) 
   {
      bonusIsleSize = 3000;
      bonusCount = rmRandInt(3, 4);
   }   
   else if(cNumberNonGaiaPlayers > 8)
   {
      bonusIsleSize = 3000;
      bonusCount = rmRandInt(3, 4);
   }   
   else
   {
      bonusIsleSize = 3000;
      bonusCount = rmRandInt(3, 4);
   }   
   rmEchoInfo("number of bonus isles "+bonusCount+ "size of isles "+bonusIsleSize);

   for(i=0; <bonusCount)
   {
      int bonusIslandID=rmCreateArea("bonus island"+i);
      rmSetAreaSize(bonusIslandID, rmAreaTilesToFraction(bonusIsleSize*0.9), rmAreaTilesToFraction(bonusIsleSize*1.1));
      rmSetAreaTerrainType(bonusIslandID, "SavannahB");
      rmAddAreaTerrainLayer(bonusIslandID, "SavannahC", 0, 6); 
      rmSetAreaWarnFailure(bonusIslandID, false);
      if(rmRandFloat(0.0, 1.0)<0.70)
         rmAddAreaConstraint(bonusIslandID, bonusIslandConstraint);
      rmAddAreaConstraint(bonusIslandID, bonusIslandEdgeConstraint);
      rmAddAreaToClass(bonusIslandID, classIsland);
      rmAddAreaToClass(bonusIslandID, classBonusIsland);
      rmAddAreaConstraint(bonusIslandID, coreBonusConstraint);
      rmSetAreaCoherence(bonusIslandID, 0.25);
      rmSetAreaSmoothDistance(bonusIslandID, 12);
      rmSetAreaHeightBlend(bonusIslandID, 2);
      rmSetAreaBaseHeight(bonusIslandID, 2.0);
      rmAddConnectionArea(extraShallowsID, bonusIslandID);
      rmAddConnectionArea(shallowsID, bonusIslandID);
   }  
*/
   rmBuildAllAreas();

   // Set up player areas.
   float playerFraction=rmAreaTilesToFraction(3500);
   for(i=1; <cNumberPlayers)
   {
      // Create the area.
      id=rmCreateArea("Player"+i);
      rmSetPlayerArea(i, id);
      rmSetAreaSize(id, 1.0, 1.0);
      rmAddAreaToClass(id, classIsland);
      rmAddAreaToClass(id, classPlayer);
      rmSetAreaWarnFailure(id, false);
      rmSetAreaMinBlobs(id, 1);
      rmSetAreaMaxBlobs(id, 5);
      rmSetAreaMinBlobDistance(id, 5.0);
      rmSetAreaMaxBlobDistance(id, 10.0);
      rmSetAreaCoherence(id, 0.6);
      rmSetAreaBaseHeight(id, 2.0);
      rmSetAreaSmoothDistance(id, 10);
      rmSetAreaHeightBlend(id, 2);
      rmAddAreaConstraint(id, islandConstraint);
      rmAddAreaConstraint(id, cornerOverlapConstraint);
      rmAddAreaConstraint(id, bonusIslandConstraint);
//      rmAddAreaConstraint(id, centerConstraint);
      rmSetAreaLocPlayer(id, i);
//      rmAddConnectionArea(extraShallowsID, id);
      rmAddConnectionArea(shallowsID, id);
      rmSetAreaTerrainType(id, "GrassA");
      rmAddAreaTerrainLayer(id, "GrassB", 0, 12);
   }

   // Build all areas
   rmBuildAllAreas();
//   rmBuildConnection(teamShallowsID);
   rmBuildConnection(shallowsID);
/*
   if(cNumberNonGaiaPlayers < 4)
      rmBuildConnection(extraShallowsID);
   else if(cNumberNonGaiaPlayers < 6)
   {   
      if(rmRandFloat(0,1)<0.5)
         rmBuildConnection(extraShallowsID);
   }
*/

   for(i=1; <cNumberPlayers*50)
   {
     // Beautification sub area.
     int id2=rmCreateArea("dirt patch"+i);
     rmSetAreaSize(id2, rmAreaTilesToFraction(20), rmAreaTilesToFraction(40));
     rmSetAreaTerrainType(id2, "GrassDirt25");
     rmSetAreaMinBlobs(id2, 1);
     rmSetAreaMaxBlobs(id2, 5);
     rmSetAreaMinBlobDistance(id2, 16.0);
     rmSetAreaMaxBlobDistance(id2, 40.0);
     rmSetAreaCoherence(id2, 0.0);
     rmAddAreaConstraint(id2, shortAvoidImpassableLand);
    rmBuildArea(id2); 
   }

   // Beautification sub area.
   for(i=1; <cNumberPlayers*15)
   {
        int id3=rmCreateArea("grass patch"+i);
        rmSetAreaSize(id3, rmAreaTilesToFraction(10), rmAreaTilesToFraction(50));
        rmSetAreaTerrainType(id3, "GrassDirt50");
        rmSetAreaMinBlobs(id3, 1);
        rmSetAreaMaxBlobs(id3, 5);
        rmSetAreaMinBlobDistance(id3, 16.0);
        rmSetAreaMaxBlobDistance(id3, 40.0);
        rmSetAreaCoherence(id3, 0.0);
        rmAddAreaConstraint(id3, shortAvoidImpassableLand);
        rmBuildArea(id3);
   } 
  
   rmPlaceObjectDefPerPlayer(startingSettlementID, true);

   // Place starting settlements.
   // Close things....
   // TC

   // Settlements.
   id=rmAddFairLoc("Settlement", false, true,  60, 80, 40, 10);

   id=rmAddFairLoc("Settlement", true, false, 70, 120, 60, 10);

   if (cNumberNonGaiaPlayers <= 4)
   {
   	id=rmAddFairLoc("Settlement", false, true,  60, 80, 40, 10);
   }
   else if (cNumberNonGaiaPlayers <= 2)
   {
   	id=rmAddFairLoc("Settlement", false, true,  60, 80, 40, 10);
   	id=rmAddFairLoc("Settlement", true, false, 70, 120, 60, 10);
   }

   if(rmPlaceFairLocs())
   {
      id=rmCreateObjectDef("far settlement2");
      rmAddObjectDefItem(id, "Settlement", 1, 0.0);
      for(i=1; <cNumberPlayers)
      {
         for(j=0; <rmGetNumberFairLocs(i))
            rmPlaceObjectDefAtLoc(id, i, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j), 1);
      }
   }

   // Towers.
   rmPlaceObjectDefPerPlayer(startingTowerID, true, 4);

    // Elev.
   int failCount=0;
   int numTries1=10*cNumberNonGaiaPlayers;
   int avoidBuildings=rmCreateTypeDistanceConstraint("avoid buildings", "Building", 10.0);
   for(i=0; <numTries1)
   {
      int elevID=rmCreateArea("elev"+i);
      rmSetAreaSize(elevID, rmAreaTilesToFraction(20), rmAreaTilesToFraction(80));
      rmSetAreaLocation(elevID, rmRandFloat(0.0, 1.0), rmRandFloat(0.0, 1.0));
      rmSetAreaWarnFailure(elevID, false); 
      rmAddAreaConstraint(elevID, avoidBuildings);
      rmAddAreaConstraint(elevID, shortAvoidImpassableLand);
      if(rmRandFloat(0.0, 1.0)<0.7)
         rmSetAreaTerrainType(elevID, "GrassA");
      rmSetAreaBaseHeight(elevID, rmRandFloat(2.0, 4.0));
      rmSetAreaHeightBlend(elevID, 1);
      rmSetAreaMinBlobs(elevID, 1);
      rmSetAreaMaxBlobs(elevID, 3);
      rmSetAreaMinBlobDistance(elevID, 16.0);
      rmSetAreaMaxBlobDistance(elevID, 40.0);
      rmSetAreaCoherence(elevID, 0.0); 

      if(rmBuildArea(elevID)==false)
      {
         // Stop trying once we fail 6 times in a row.
         failCount++;
         if(failCount==6)
            break;
      }
      else
         failCount=0; 
   }

   // Text
   rmSetStatusText("",0.40);

   // Straggler trees.
   rmPlaceObjectDefPerPlayer(stragglerTreeID, false, rmRandInt(0, 7));

   // Gold
   rmPlaceObjectDefPerPlayer(startingGoldID, false);

    // Goats
   rmPlaceObjectDefPerPlayer(closeGoatsID, true);

   // Deer
   rmPlaceObjectDefPerPlayer(closeDeerID, false); 

   // Elk.
   rmPlaceObjectDefPerPlayer(closeElkID, false);

   // Medium things....
   // Gold
   for(i=1; <cNumberPlayers)
      rmPlaceObjectDefInArea(mediumGoldID, 0, rmAreaID("player"+i));

   // Goats
   for(i=1; <cNumberPlayers)
      rmPlaceObjectDefInArea(mediumGoatsID, 0, rmAreaID("player"+i));

   // elk or deer.  Flip a coin.
   if(rmRandFloat(0.0, 1.0)<0.5)   
   {
      for(i=1; <cNumberPlayers)
         rmPlaceObjectDefInArea(mediumElkID, 0, rmAreaID("player"+i));
   }
   else if(rmRandFloat(0,1)<0.2)
   {
      for(i=1; <cNumberPlayers)
      {
        rmPlaceObjectDefInArea(mediumElkID, 0, rmAreaID("player"+i));
        rmPlaceObjectDefInArea(mediumDeerID, 0, rmAreaID("player"+i));
      }
   }
   else
   {
      for(i=1; <cNumberPlayers)
         rmPlaceObjectDefInArea(mediumDeerID, 0, rmAreaID("player"+i));
   }

      
   // Far things.

   // Player Far Gold, need goldNum since it randomizes for each i
   int goldNum = rmRandInt(1,2);
   for(i=1; <cNumberPlayers)
      rmPlaceObjectDefInArea(farGoldID, false, rmAreaID("player"+i), goldNum);
  
  // Bonus Gold
   //rmPlaceObjectDefPerPlayer(bonusGoldID, false, rmRandInt(1,2));

   // Relics.
   for(i=1; <cNumberPlayers)
      rmPlaceObjectDefInArea(relicID, i, rmAreaID("player"+i));

   // Hawks
   rmPlaceObjectDefPerPlayer(farhawkID, false, 2);

   for(i=1; <cNumberPlayers)
      rmPlaceObjectDefInArea(farChickenID, false, rmAreaID("player"+i));
/*
   // Bonus huntable
   for(i=1; <cNumberPlayers)
      rmPlaceObjectDefInArea(bonusHuntableID, false, rmAreaID("player"+i));

    // Bonus huntable
   for(i=1; <bonusCount)
   {
      rmPlaceObjectDefInArea(bonusHuntableID2, false, rmAreaID("bonus island"+i));
      rmPlaceObjectDefInArea(bonusHuntableID3, false, rmAreaID("bonus island"+i));
   }
*/   
   // Goats
   int goatNum = rmRandInt(1,2);
   for(i=1; <cNumberPlayers)
      rmPlaceObjectDefInArea(farGoatsID, false, rmAreaID("player"+i), goatNum);

   // Predators
   rmPlaceObjectDefPerPlayer(farPredatorID, false, 2);

   // Text
   rmSetStatusText("",0.60);

   // Random trees.
   rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers);

   rmPlaceObjectDefAtLoc(randomTreeID2, 0, 0.5, 0.5, 2*cNumberNonGaiaPlayers);

   /// Forest.
   int classForest=rmDefineClass("forest");
   int forestObjConstraint=rmCreateTypeDistanceConstraint("forest obj", "all", 6.0);
   int forestConstraint=rmCreateClassDistanceConstraint("forest v forest", rmClassID("forest"), 16.0);
   int forestSettleConstraint=rmCreateClassDistanceConstraint("forest settle", rmClassID("starting settlement"), 20.0);
   int forestCount=10*cNumberNonGaiaPlayers;
   failCount=0;
   for(i=0; <forestCount)
   {
      int forestID=rmCreateArea("forest"+i);
      rmSetAreaSize(forestID, rmAreaTilesToFraction(150), rmAreaTilesToFraction(300));
      rmSetAreaWarnFailure(forestID, false);
      if(rmRandFloat(0,1)<0.8)
         rmSetAreaForestType(forestID, "mixed oak forest");
      else
         rmSetAreaForestType(forestID, "pine forest");
      rmAddAreaConstraint(forestID, forestSettleConstraint);
      rmAddAreaConstraint(forestID, forestObjConstraint);
      rmAddAreaConstraint(forestID, forestConstraint);
      rmAddAreaConstraint(forestID, avoidImpassableLand);
      rmAddAreaToClass(forestID, classForest);
      
      rmSetAreaMinBlobs(forestID, 2);
      rmSetAreaMaxBlobs(forestID, 4);
      rmSetAreaMinBlobDistance(forestID, 16.0);
      rmSetAreaMaxBlobDistance(forestID, 20.0);
      rmSetAreaCoherence(forestID, 0.0);

      // Hill trees?
      if(rmRandFloat(0.0, 1.0)<0.2)
         rmSetAreaBaseHeight(forestID, rmRandFloat(3.0, 4.0));

      if(rmBuildArea(forestID)==false)
      {
         // Stop trying once we fail 3 times in a row.
         failCount++;
         if(failCount==3)
            break;
      }
      else
         failCount=0;
   }

   // Text
   rmSetStatusText("",0.80);

   // Text
   rmSetStatusText("",0.90);

   int skeletonID=rmCreateObjectDef("dead animal");
   rmAddObjectDefItem(skeletonID, "skeleton animal", 1, 0.0);
   rmSetObjectDefMinDistance(skeletonID, 0.0);
   rmSetObjectDefMaxDistance(skeletonID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(skeletonID, avoidAll);
   rmAddObjectDefConstraint(skeletonID, avoidImpassableLand);
   rmPlaceObjectDefAtLoc(skeletonID, 0, 0.5, 0.5, 3*cNumberNonGaiaPlayers); 

   int lilyID=rmCreateObjectDef("lily pads");
   rmAddObjectDefItem(lilyID, "water lilly", 1, 0.0);
   rmSetObjectDefMinDistance(lilyID, 0.0);
   rmSetObjectDefMaxDistance(lilyID, 600);
   rmAddObjectDefConstraint(lilyID, avoidAll);
   rmPlaceObjectDefAtLoc(lilyID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers); 

   int lily2ID=rmCreateObjectDef("lily2 pad groups");
   rmAddObjectDefItem(lily2ID, "water lilly", 4, 2.0);
   rmSetObjectDefMinDistance(lily2ID, 0.0);
   rmSetObjectDefMaxDistance(lily2ID, 600);
   rmAddObjectDefConstraint(lily2ID, avoidAll);
   rmPlaceObjectDefAtLoc(lily2ID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers); 

   int decorID=rmCreateObjectDef("water decorations");
   rmAddObjectDefItem(decorID, "water decoration", 3, 6.0);
   rmSetObjectDefMinDistance(decorID, 0.0);
   rmSetObjectDefMaxDistance(decorID, 600);
   rmAddObjectDefConstraint(decorID, avoidAll);
   rmPlaceObjectDefAtLoc(decorID, 0, 0.5, 0.5, 10*cNumberNonGaiaPlayers);

  // Text
   rmSetStatusText("",1.0);

}  




